﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.Diagrams;
using Telerik.Windows.Controls.Diagrams.Extensions;
using Telerik.Windows.Diagrams.Core;

namespace DiagramResizing
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static MainWindow()
        {
            var saveBinding = new CommandBinding(DiagramCommands.Save, ExecuteSave, CanExecuteSave);
            var openBinding = new CommandBinding(DiagramCommands.Open, ExecuteOpen, CanExecuteOpen);

            CommandManager.RegisterClassCommandBinding(typeof(MainWindow), saveBinding);
            CommandManager.RegisterClassCommandBinding(typeof(MainWindow), openBinding);
        }

        public MainWindow()
        {
            InitializeComponent();

            this.toolBox.ItemsSource = new HierarchicalGalleryItemsCollection();
            this.diagram.ServiceLocator.Register<IResizingService>(new CustomResizingService(this.diagram));
        }

        private static void CanExecuteSave(object sender, CanExecuteRoutedEventArgs e)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");
            var owner = sender as MainWindow;
            e.CanExecute = owner != null && owner.diagram != null;
        }

        private static void CanExecuteOpen(object sender, CanExecuteRoutedEventArgs e)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");
            var owner = sender as MainWindow;
            e.CanExecute = owner != null && owner.diagram != null;
        }

        private static void ExecuteSave(object sender, ExecutedRoutedEventArgs e)
        {
            var owner = sender as MainWindow;
            if (owner != null)
            {
                var dialog = new SaveFileDialog
                {
                    DefaultExt = "xml",
                    Filter = "Diagram files|*.xml|All Files (*.*)|*.*",
                    FileName = "Diagram.xml"
                };
                if (dialog.ShowDialog() == true)
                {
                    using (var fileStream = dialog.OpenFile())
                    {
                        var serializationString = owner.diagram.Save();
                        using (var writer = new StreamWriter(fileStream))
                        {
                            writer.Write(serializationString);
                        }
                    }
                }
            }
        }

        private static void ExecuteOpen(object sender, ExecutedRoutedEventArgs e)
        {
            var owner = sender as MainWindow;
            if (owner != null)
            {
                var dialog = new OpenFileDialog { Multiselect = false, Filter = "Diagram (.xml)|*.xml|All Files (*.*)|*.*" };
                if (dialog.ShowDialog() == true)
                {
                    owner.diagram.Clear();
                    using (var fileStream = File.OpenRead(dialog.FileName))
                    {
                        var reader = new StreamReader(fileStream);
                        var serializationString = reader.ReadToEnd();
                        owner.diagram.Load(serializationString);
                    }
                }
            }
        }
    }
}
